;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xda59b5fdab501bea3d664cd4bfdf3804 . "./code.cg2js/cg-acache.fasl")
(#x7a8058f9253714881d07c7fe5fca98fe . "./code.cg2js/cg-base.fasl")
(#xe861c578db42bde4a24c0d2f1419fe56 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x670c65f60712c847c09e1cbf4b52da9c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x3d9a2db148e4266c6ba34a9bbe3f0644 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xdb839765e763e1037d456ffc7314babc . "./code.cg2js/cg-button.fasl")
(#x4577f0cb57319b6e10b4d3f873013dcc . "./code.cg2js/cg-calendar.fasl")
(#x72a2ccc3083f5b0a9086435077840c66 . "./code.cg2js/cg-caret.fasl")
(#x1dc6dbf7c8d71addca666d9e92887315 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xc6e9e0c822912513e7082eccbacfefb7 . "./code.cg2js/cg-chart-widget.fasl")
(#x01d26f1de2175aa796f3e69d3799ab18 . "./code.cg2js/cg-check-box.fasl")
(#xfe7d7a5ed29b164270935a3ccae65743 . "./code.cg2js/cg-choice-list.fasl")
(#x81ae87888fdcd8e1e5a3ac16a94aba1f . "./code.cg2js/cg-choose-printer.fasl")
(#x1b7fb17cd9fb9746b56e6cb0f0b16b94 . "./code.cg2js/cg-class-grid.fasl")
(#x557f0d68bec7f493fd17ab608e2af40e . "./code.cg2js/cg-class-slot-grid.fasl")
(#x09d00ebe53336350ee2c4b676e086e73 . "./code.cg2js/cg-class-support.fasl")
(#xd2edeae654a9cd8beb0373fae52da49c . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x89dc80f67f6116b903a109a5dfa9ab12 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x46252ae09055d9c83b94545c6b9e20da . "./code.cg2js/cg-clipboard.fasl")
(#xf68c89fe3b809f8b8385323180bb9712 . "./code.cg2js/cg-color-dialog.fasl")
(#xcc40a2392021ca3d11a1d5a889c20da4 . "./code.cg2js/cg-combo-box.fasl")
(#x81b1599303ab789eb68f73f975e7329c . "./code.cg2js/cg-common-control.fasl")
(#xfe1e871ed1bb71815f9dddcb98f19e8c . "./code.cg2js/cg-comtab.fasl")
(#x4510049cad1e0baf2d58103934f534fc . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xc0b9a909386614b73683d784746c08d6 . "./code.cg2js/cg-curve.fasl")
(#x678b358abb45b0a3f2ec786258ca02fa . "./code.cg2js/cg-dde-utils.fasl")
(#x705cac5ac244aac4785ea13affd93c3f . "./code.cg2js/cg-dev.fasl")
(#xe8cba42161efdb303561faa417d9eab1 . "./code.cg2js/cg-dialog-item.fasl")
(#x08a0803377fbe071a443b6dc3a6e6232 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x5c45dde14d4ea57dc2fdfca50b68f985 . "./code.cg2js/cg-directory-dialog.fasl")
(#xa485240face83157b00bad2b86c7f2df . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x5808b9a9267f07f0cfb574ab308bb06e . "./code.cg2js/cg-drag-and-drop.fasl")
(#xae3cbf500eb56ede3846a04c908f7ed7 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x5887f2b7ad0ee3044df0e925d4a32738 . "./code.cg2js/cg-drawable.fasl")
(#xebef348804e0305e1aa942def41f2139 . "./code.cg2js/cg-dropping-outline.fasl")
(#x3da105b206b10fb5252fabccf0732ece . "./code.cg2js/cg-edit-in-place.fasl")
(#x9b3fa2bb51b6e6ee5b403ff4b7f7b245 . "./code.cg2js/cg-editable-text.fasl")
(#xfe0c7be7d96c10518dd38643df8377f0 . "./code.cg2js/cg-file-dialog.fasl")
(#x945bffe2f3010f734ddb777224c6929c . "./code.cg2js/cg-fill-texture.fasl")
(#xcafdb2fb58f50efb110576fed8450c34 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x486e1e1b0a1107c96df4121c4d180824 . "./code.cg2js/cg-font-dialog.fasl")
(#x3f4cd661b8a4801894653088fc1f0cb2 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x9571a91559222e134bf230bedb60e0be . "./code.cg2js/cg-get-pixmap.fasl")
(#x36049755296332ade35ee7e7b3a842d5 . "./code.cg2js/cg-get-position.fasl")
(#x0c081fad5856d8dc14342623ea48e249 . "./code.cg2js/cg-graphics-context.fasl")
(#x617c81498ed3dc7c9319046f77ad3e1e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x9c2d163ea6e5624bb5f951f99904727b . "./code.cg2js/cg-grid-widget.fasl")
(#xab7758eaea131b3d2d578101696a49d6 . "./code.cg2js/cg-group-box.fasl")
(#x47aad4b3f1d6c7f186217647356ae32b . "./code.cg2js/cg-header-control.fasl")
(#x1e96cde96ffe41d91742ec1f6e6bb4de . "./code.cg2js/cg-hotspot.fasl")
(#x5e4a1d45c4bc3d43ea6f9422b866eae4 . "./code.cg2js/cg-html-dialog.fasl")
(#x333f15e1b9c8e0f03dd445dc4a0e6a1a . "./code.cg2js/cg-html-widget.fasl")
(#x73f3f6a6210a77a6fc661f42c81781f8 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x510ee4e08c9a3faf6fba3053ebdf017c . "./code.cg2js/cg-icon.fasl")
(#xcfa30e2f236ddc91b43b7291c3adaa1e . "./code.cg2js/cg-ie.fasl")
(#x579e61025ea69efc9f7606f4662582c3 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x361852e6517822cb1a7d1cc799612821 . "./code.cg2js/cg-intersect.fasl")
(#xfb1d509704dfb044088478f12af69ada . "./code.cg2js/cg-item-list.fasl")
(#xa0c7202fdff2ae80f4ae498a2e7b3072 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x961b35c84d98f019bdb24bd24815cb29 . "./code.cg2js/cg-lamp.fasl")
(#x621f491f53aaf9b077dced1a137f6069 . "./code.cg2js/cg-layout.fasl")
(#x18bbbdb59bd7169f067e89b42b168f86 . "./code.cg2js/cg-lettered-menu.fasl")
(#xffa6172cc3652645d3e8d95d93763d8a . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9f04b06c393a537e342d9defad423e34 . "./code.cg2js/cg-lisp-text.fasl")
(#x867ef0533c927f609e81ce4e7df3cc7a . "./code.cg2js/cg-lisp-widget.fasl")
(#xb6f5ba4422f2f6388716f5848912b2fa . "./code.cg2js/cg-list-view.fasl")
(#x390ba8584b504c84c48c6b5e377b7517 . "./code.cg2js/cg-mci.fasl")
(#xd19d050204c1401923d6f4d0bdb0fb6e . "./code.cg2js/cg-menu-tooltip.fasl")
(#x92a2b2864f600dae7f58327cbe2497a6 . "./code.cg2js/cg-menu.fasl")
(#xaa2dd378b9c9220e1d938c227d9108c4 . "./code.cg2js/cg-message-dialog.fasl")
(#xfdb8728ae30fa14928ec73ca4b4b87b4 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xa5ddc4d3d238ebecba5f5dcaad1b6016 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x5e03c22df3e296945fb7d81b165cd876 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xf6b3501fc50344742689efbaeae05f0e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4e9df8f7688d07b28dea4340282cf9be . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe8d576a5f7242fb9b0e38e4a7946fc47 . "./code.cg2js/cg-nodes.fasl")
(#x313d31de04521733f44985d958767330 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x75a219f5d688ee6c13278680520a175d . "./code.cg2js/cg-object-editor.fasl")
(#x08a70cc12539980510573bef77e2723c . "./code.cg2js/cg-ocx.fasl")
(#xec05ddfe0003362f1710026dfa157c9f . "./code.cg2js/cg-os-widget.fasl")
(#x4cd9f907696d6b5e08c358824a03098b . "./code.cg2js/cg-os-window.fasl")
(#x2dbbac8756d57bef633a27c4336d97cf . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xb02e220e027d00e798a691ccfce8c5fa . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7fdf9ef0112b93d37088873cacc47be6 . "./code.cg2js/cg-outline.fasl")
(#x2cb9856d7a358b4bf5c219d7429f66c0 . "./code.cg2js/cg-palette.fasl")
(#xf92a95e8bf443aff280a34361dc9fe78 . "./code.cg2js/cg-paren-matching.fasl")
(#xb86790ae216ff6e6ec829f5f20752862 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd4ff10308cee5c77eb9f29396085dc3e . "./code.cg2js/cg-picture-widget.fasl")
(#xca8c15f2c3eaf3741f7bab4ef1bdc1a6 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x0d98eeb7037fa274fe9f178e5044664c . "./code.cg2js/cg-pixmap-printing.fasl")
(#x6ac45687133b4753d9bacf22550c43dc . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x6ff25f2954c0811ff8a52d4111d83d47 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x021316c7db643ab892f03ee2352165d2 . "./code.cg2js/cg-pixmap.fasl")
(#x194f1b3b97e784a20a7fc7216878e4e9 . "./code.cg2js/cg-player.fasl")
(#x683ae1febe15e9fc012dfcca32e9f040 . "./code.cg2js/cg-plot-widget.fasl")
(#x28d7f3b6123dc4864ba0a547cd8c0d88 . "./code.cg2js/cg-printing.fasl")
(#xb9afa0e2637f086b22a4aee8330a18fa . "./code.cg2js/cg-progress-indicator.fasl")
(#xfec7ebb4984ce2cce7f4d680361e32da . "./code.cg2js/cg-project-window.fasl")
(#xd2971f613b28176fc2c7e4d32f5e57c1 . "./code.cg2js/cg-property.fasl")
(#x7ac26c252981e9214b76626b98f29466 . "./code.cg2js/cg-radio-button.fasl")
(#x14bb2efbc04f8ba16bd479b96684e1c6 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x28c8839d5a499de6b078e4802bf35147 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x622e93b111851931a0daf7dc1dc12d09 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x187841fa24fef7b7839ca94d62bb2658 . "./code.cg2js/cg-rich-edit.fasl")
(#xd2f697de3413070f874f6224e34552e8 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x1e4fb9925fb1af53c7afd52fd99f3f0c . "./code.cg2js/cg-scaling-stream.fasl")
(#xb1babbc33ee187f0e7b2eadae19b534e . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x51a3d278f51be285cee5035bfc312a0f . "./code.cg2js/cg-scroll-bar.fasl")
(#x4b694e37be2fc48f7e727a8251b16637 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x072f3eed6b4b24c86d8ae479190e0502 . "./code.cg2js/cg-selected-object.fasl")
(#xef34be202ab08f3d8f90486e570770ff . "./code.cg2js/cg-shortcut-menu.fasl")
(#x2ce3dd1da7f086db22e42ee5f6732011 . "./code.cg2js/cg-split-bar.fasl")
(#xa00fe8e9b677a4226508d81dfa437abb . "./code.cg2js/cg-static-text.fasl")
(#x3b7d44016796723053b235d7b5a5ff44 . "./code.cg2js/cg-status-bar.fasl")
(#xead618a537e59d3e29c03f8d11dbf2db . "./code.cg2js/cg-string-dialog.fasl")
(#x57d2d647b32bfee16aab4411da542afe . "./code.cg2js/cg-tab-control.fasl")
(#x22284eb24f4406527586645bdd842f26 . "./code.cg2js/cg-template-string.fasl")
(#x9de97d62f4648dc4e8b989a786eeb47f . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x0e6d2bea5f78edff1cb5bc33a1846a04 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xae18810ba8730576241100a3a79eaaa2 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x1009770dbee8f1dbb62e8589035124c8 . "./code.cg2js/cg-text-or-combo.fasl")
(#xf384ef28e7a0ca6b85ba90824cdbf919 . "./code.cg2js/cg-text-widget.fasl")
(#x62ddf41287d27a5385f2effc884af2b0 . "./code.cg2js/cg-timer.fasl")
(#xdb02c124435f6ceb78772cf97a47f119 . "./code.cg2js/cg-toggling-widget.fasl")
(#x4849960b685c1be3a6f947963c588855 . "./code.cg2js/cg-toolbar.fasl")
(#xb21ebef5b9f4b0772f717f8f7819dc9e . "./code.cg2js/cg-tooltip.fasl")
(#x176825bf2f4ce6d9e2d3d58948c731b3 . "./code.cg2js/cg-touch.fasl")
(#xc92908e7cfbe6c153ad05da4ee34a59c . "./code.cg2js/cg-trackbar.fasl")
(#x36da07a75ba46451f37882a4daa27378 . "./code.cg2js/cg-tray.fasl")
(#xb9c487dc5d911fdb3b2fbbbbac8b12ee . "./code.cg2js/cg-up-down-control.fasl")
(#x78662cf7a345bebb8729250c3be4518c . "./code.cg2js/cg-utility-dialog.fasl")
(#x132f6b24b930e00b31da98ef7ec16808 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x59e545d3962b7a9eed3280b0b8c9bce0 . "./code.cg2js/cg-web-browser.fasl")
(#xb0da5617d3d12eade71dd272fa83abd4 . "./code.cg2js/cg-wrap-string.fasl")
(#xb1a68a3a7d35fd41a5ff2c4f1484474f . "./code.cg2js/cg-yes-no-list.fasl")
(#xf44f1546dd18e1938380124671b1e188 . "./code.cg2js/cg.fasl")
(#xedf247d6b93da78d9e9069899c5f8046 . "./code.cg2js/cg-yes-no-string.fasl")
(#xfb310a140df9edc03558ca50579513d3 . "./code.idejs/ide-acache.fasl")
(#x3be2f564136e116c4f9f859b97fb099f . "./code.idejs/ide-apropos.fasl")
(#xf7087e1d6ca82920dfc079beb00cecaf . "./code.idejs/ide-base.fasl")
(#x28e869d6acc4e5faf47813f125add7b0 . "./code.idejs/ide-class-browser.fasl")
(#x3cf007edee9776e728bb129d640a3815 . "./code.idejs/ide-class-grapher.fasl")
(#xeb2ea537815ed9cbe2d3409520c639d4 . "./code.idejs/ide-clipboard.fasl")
(#x5e2a7d9d27c39b69c3a7c78febf9af1d . "./code.idejs/ide-dev.fasl")
(#x5dc3795ec2d642d67e46eaefeecff7c6 . "./code.idejs/ide-editor.fasl")
(#x94593278f14585f1628f9ff92e83897f . "./code.idejs/ide-find-definitions.fasl")
(#x5f9c584f4e969ce84975d7fb5681e943 . "./code.idejs/ide-find-in-files.fasl")
(#x1d97ff05042c3d7d57025b4037587afb . "./code.idejs/ide-form.fasl")
(#x87b0f6bb5b89907de3f270e8864bc050 . "./code.idejs/ide-grapher.fasl")
(#xe2d06f9049db2936d40bdd1255503f2f . "./code.idejs/ide-inspector.fasl")
(#x8628ab847f0ba02dbf284fbf784e7501 . "./code.idejs/ide-intro.fasl")
(#x1d798e0664fe23da299443bb7568cc67 . "./code.idejs/ide-keyshort.fasl")
(#xcdbd14f8bb2d9dcb1794167ab7c9f3a4 . "./code.idejs/ide-navigator.fasl")
(#x028afb4884156f8edbfa190f3ace17f4 . "./code.idejs/ide-package-list.fasl")
(#x972aad417513c7545d08912d228899da . "./code.idejs/ide-patch-dialog.fasl")
(#x7d4dfb952eb9c729ca4c6f5575d1ea13 . "./code.idejs/ide-process-browser.fasl")
(#x172cc41e2ea0944b366fd032b37d444d . "./code.idejs/ide-project.fasl")
(#x8af2215da5872416301697ec7d01ea61 . "./code.idejs/ide-trace.fasl")
(#x66b9964c5878803fbffe386d690ed3e5 . "./code.idejs/ide-window-list.fasl")
(#x37bb98aeb287510da8b6f41895b05710 . "./code.idejs/ide.fasl")
(#x68149b79128815fb04933c4813dbe32f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
